@@ -97,6 +97,7 @@ struct _NautilusWindow
     AdwApplicationWindow parent_instance;
 
     AdwTabView *tab_view;
+    AdwTabBar *tab_bar;
     AdwTabPage *menu_page;
 
     GList *slots;
@@ -191,6 +192,17 @@ action_close_current_view (GSimpleAction *action,
     adw_tab_view_close_page (window->tab_view, page);
 }
 
+static void
+action_close_other_tabs (GSimpleAction *action,
+                         GVariant      *parameters,
+                         gpointer       user_data)
+{
+    NautilusWindow *window = user_data;
+    AdwTabPage *page = get_current_page (window);
+
+    adw_tab_view_close_other_pages (window->tab_view, page);
+}
+
 static void
 action_go_home (GSimpleAction *action,
                 GVariant      *state,
@@ -427,19 +439,6 @@ action_undo (GSimpleAction *action,
     nautilus_file_undo_manager_undo (GTK_WINDOW (window), NULL);
 }
 
-static void
-action_toggle_state_view_button (GSimpleAction *action,
-                                 GVariant      *state,
-                                 gpointer       user_data)
-{
-    GVariant *current_state;
-
-    current_state = g_action_get_state (G_ACTION (action));
-    g_action_change_state (G_ACTION (action),
-                           g_variant_new_boolean (!g_variant_get_boolean (current_state)));
-    g_variant_unref (current_state);
-}
-
 static void
 action_show_current_location_menu (GSimpleAction *action,
                                    GVariant      *state,
@@ -491,9 +490,13 @@ tab_view_setup_menu_cb (AdwTabView     *tab_view,
 {
     GAction *move_tab_left_action;
     GAction *move_tab_right_action;
+    GAction *restore_tab_action;
     int position, n_pages;
+    gboolean menu_is_closed = (page == NULL);
 
-    if (page != NULL)
+    window->menu_page = page;
+
+    if (!menu_is_closed)
     {
         position = adw_tab_view_get_page_position (tab_view, page);
         n_pages = adw_tab_view_get_n_pages (tab_view);
@@ -503,13 +506,16 @@ tab_view_setup_menu_cb (AdwTabView     *tab_view,
                                                        "tab-move-left");
     move_tab_right_action = g_action_map_lookup_action (G_ACTION_MAP (window),
                                                         "tab-move-right");
+    restore_tab_action = g_action_map_lookup_action (G_ACTION_MAP (window),
+                                                     "restore-tab");
 
+    /* Re-enable all of the actions if the menu is closed */
     g_simple_action_set_enabled (G_SIMPLE_ACTION (move_tab_left_action),
-                                 page == NULL || position > 0);
+                                 menu_is_closed || position > 0);
     g_simple_action_set_enabled (G_SIMPLE_ACTION (move_tab_right_action),
-                                 page == NULL || position < n_pages - 1);
-
-    window->menu_page = page;
+                                 menu_is_closed || position < n_pages - 1);
+    g_simple_action_set_enabled (G_SIMPLE_ACTION (restore_tab_action),
+                                 menu_is_closed || g_queue_get_length (window->tab_data_queue) > 0);
 }
 
 static void
@@ -964,11 +970,6 @@ places_sidebar_drag_perform_drop_cb (NautilusGtkPlacesSidebar *sidebar,
     char *dest_uri;
     GList *source_uri_list;
 
-    if (source_file_list == NULL)
-    {
-        return;
-    }
-
     dest_uri = g_file_get_uri (dest_file);
     source_uri_list = build_uri_list_from_gfile_list (source_file_list);
 
@@ -1312,6 +1313,7 @@ nautilus_window_show_operation_notification (NautilusWindow *window,
 
     toast = adw_toast_new (main_label);
     adw_toast_set_priority (toast, ADW_TOAST_PRIORITY_HIGH);
+    adw_toast_set_use_markup (toast, FALSE);
 
     current_location = nautilus_window_slot_get_location (window->active_slot);
     if (gtk_window_is_active (GTK_WINDOW (window)))
@@ -1440,6 +1442,18 @@ tab_view_create_window_cb (AdwTabView     *tab_view,
     return new_window->tab_view;
 }
 
+static void
+action_tab_move_new_window (GSimpleAction *action,
+                            GVariant      *parameter,
+                            gpointer       user_data)
+{
+    NautilusWindow *window = user_data;
+    AdwTabPage *page = get_current_page (window);
+    AdwTabView *new_view = tab_view_create_window_cb (window->tab_view, window);
+
+    adw_tab_view_transfer_page (window->tab_view, page, new_view, 0);
+}
+
 static void
 setup_tab_view (NautilusWindow *window)
 {
@@ -1463,6 +1477,55 @@ setup_tab_view (NautilusWindow *window)
                       window);
 }
 
+static GdkDragAction
+extra_drag_value_cb (AdwTabBar    *self,
+                     AdwTabPage   *page,
+                     const GValue *value,
+                     gpointer      user_data)
+{
+    NautilusWindowSlot *slot = NAUTILUS_WINDOW_SLOT (adw_tab_page_get_child (page));
+    g_autoptr (NautilusFile) file = nautilus_file_get (nautilus_window_slot_get_location (slot));
+    GdkDragAction action = 0;
+
+    if (value != NULL)
+    {
+        if (G_VALUE_HOLDS (value, GDK_TYPE_FILE_LIST))
+        {
+            GSList *file_list = g_value_get_boxed (value);
+            if (file_list != NULL)
+            {
+                action = nautilus_dnd_get_preferred_action (file, G_FILE (file_list->data));
+            }
+        }
+        else if (G_VALUE_HOLDS (value, G_TYPE_STRING))
+        {
+            action = GDK_ACTION_COPY;
+        }
+    }
+
+    /* We set the preferred action on the drop from the results of this function,
+     * but since we don't have access to the GtkDropTarget, we can't get the preferred
+     * action in ::drop, so let's set it as data on the page. We probably should
+     * expose the preferred action within libadwaita.
+     */
+    g_object_set_data (G_OBJECT (page), "drag-action", GINT_TO_POINTER (action));
+    return action;
+}
+
+static gboolean
+extra_drag_drop_cb (AdwTabBar    *self,
+                    AdwTabPage   *page,
+                    const GValue *value,
+                    gpointer      user_data)
+{
+    NautilusWindowSlot *slot = NAUTILUS_WINDOW_SLOT (adw_tab_page_get_child (page));
+    NautilusFilesView *view = NAUTILUS_FILES_VIEW (nautilus_window_slot_get_current_view (slot));
+    GFile *target_location = nautilus_window_slot_get_location (slot);
+    GdkDragAction action = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (page), "drag-action"));
+
+    return nautilus_dnd_perform_drop (view, value, action, target_location);
+}
+
 const GActionEntry win_entries[] =
 {
     { "back", action_back },
@@ -1470,7 +1533,6 @@ const GActionEntry win_entries[] =
     { "back-n", action_back_n, "u" },
     { "forward-n", action_forward_n, "u" },
     { "up", action_up },
-    { "view-menu", action_toggle_state_view_button, NULL, "false", NULL },
     { "current-location-menu", action_show_current_location_menu },
     { "open-location", action_open_location, "s" },
     { "reload", action_reload },
@@ -1482,10 +1544,12 @@ const GActionEntry win_entries[] =
     { "redo", action_redo },
     /* Only accesible by shorcuts */
     { "close-current-view", action_close_current_view },
+    { "close-other-tabs", action_close_other_tabs },
     { "go-home", action_go_home },
     { "go-starred", action_go_starred },
     { "tab-move-left", action_tab_move_left },
     { "tab-move-right", action_tab_move_right },
+    { "tab-move-new-window", action_tab_move_new_window },
     { "prompt-root-location", action_prompt_for_location_root },
     { "prompt-home-location", action_prompt_for_location_home },
     { "go-to-tab", NULL, "i", "0", action_go_to_tab },
@@ -1573,6 +1637,19 @@ nautilus_window_constructed (GObject *self)
                                  NAUTILUS_WINDOW_DEFAULT_HEIGHT);
 
     setup_tab_view (window);
+
+    /* Lets not support GDK_ACTION_LINK for now.  This only impacts x11
+     * and on x11 we are using a hack in list-base to handle GDK_ACTION_LINK
+     * which we can not replicate here because we don't have access to the
+     * GtkDropTarget to check the actions on the GdkDrag.
+     */
+    adw_tab_bar_setup_extra_drop_target (window->tab_bar,
+                                         GDK_ACTION_COPY | GDK_ACTION_MOVE,
+                                         (GType [2]) {GDK_TYPE_FILE_LIST, G_TYPE_STRING}, 2);
+    adw_tab_bar_set_extra_drag_preload (window->tab_bar, TRUE);
+    g_signal_connect (window->tab_bar, "extra-drag-value", G_CALLBACK (extra_drag_value_cb), NULL);
+    g_signal_connect (window->tab_bar, "extra-drag-drop", G_CALLBACK (extra_drag_drop_cb), NULL);
+
     nautilus_window_set_up_sidebar (window);
 
 
@@ -2101,8 +2178,7 @@ nautilus_window_init (NautilusWindow *window)
     window->slots = NULL;
     window->active_slot = NULL;
 
-    gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (window)),
-                                 "nautilus-window");
+    gtk_widget_add_css_class (GTK_WIDGET (window), "nautilus-window");
 
     window_group = gtk_window_group_new ();
     gtk_window_group_add_window (window_group, GTK_WINDOW (window));
@@ -2223,6 +2299,7 @@ nautilus_window_class_init (NautilusWindowClass *class)
     gtk_widget_class_bind_template_child (wclass, NautilusWindow, places_sidebar);
     gtk_widget_class_bind_template_child (wclass, NautilusWindow, toast_overlay);
     gtk_widget_class_bind_template_child (wclass, NautilusWindow, tab_view);
+    gtk_widget_class_bind_template_child (wclass, NautilusWindow, tab_bar);
 
     signals[SLOT_ADDED] =
         g_signal_new ("slot-added",
@@ -2307,6 +2384,7 @@ nautilus_window_show_about_dialog (NautilusWindow *window)
                            "version", VERSION,
                            "website", "https://wiki.gnome.org/action/show/Apps/Files",
                            "issue-url", "https://gitlab.gnome.org/GNOME/nautilus/-/issues/new",
+                           "support-url", "https://discourse.gnome.org/tag/nautilus",
                            "debug-info", debug_info,
                            "copyright", "© 1999 The Files Authors",
                            "license-type", GTK_LICENSE_GPL_3_0,
